/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
  
 * Created on November 1, 2002, 1:15 PM
 */

package org.kablog.kdb;
import javax.microedition.rms.*;
import java.io.*;


/**
 *
 * @author  Todd C. Stellanova
 */
public abstract class KDbAccessor {
    
    protected static KDbAccessor dbCache;
    
    private static final boolean bDebug = false;
    
    public String storeName;
    protected RecordStore dbStore;
    protected RecordEnumeration dbEnum;
    protected RecordComparator dbComparator;

    public static final int INVALID_REC_ID = -666;
    protected int curRecID = INVALID_REC_ID;

    /**
     * Create a template blank record. Implemented by subclasses.
     */
   protected abstract KDbRecord createBlankRecord();
   
     /**
      * @return True if prefs exist, false otherwise.
      */
     public boolean dbExists() {
         boolean bExists = false;
         
        if (dbStore == null) {
            // if the store appears in this list, does that mean it has
            // already been created...or just that it *could* be created?
            // this code assumes it has already been created
            String[] storesList = RecordStore.listRecordStores();
            if (storesList != null) {
               for (int i = 0; i < storesList.length; i++) {
                   if (storesList[i].equals(storeName)) {
                       bExists = true;
                       break;
                   }                       
               }
            }
        }
        else //if dbStore is non-null, then the db MUST exist
            bExists = true;
        
        return bExists;
     }//dbExists
     
     /**
      * @return The number of entries in the database. Can be zero!
      */
     public int getEntryCount() {
         int count = 0;
         if (dbExists())
             try {
                 getDbEnumeration();
                 count = dbEnum.numRecords();
             }
             catch (KDbException dbEx) {
                 if (bDebug) System.err.println("getDbEnumeration threw: " + dbEx);
             }
         return count;
     }//getEntryCount
     
     /**
      * Open the database and provide an enumeration of all entries.
      */ 
     protected RecordEnumeration getDbEnumeration() throws KDbException {
         if (dbEnum == null) {
             if (openDB()) {
                 //dbStore.enumerateRecords(RecordFilter filter, RecordComparator comparator, boolean keepUpdated);
                 try {
                     dbEnum = dbStore.enumerateRecords(null,dbComparator,true);
                 }
                 catch (RecordStoreNotOpenException storeClosedEx) {
                    if (bDebug) System.err.println("Tried to enumerate closed recStore: " + storeClosedEx); 
                 }
                 catch (Throwable anyThrow) {
                    if (bDebug) System.err.println("Enumerating db failed: " + anyThrow); 
                 }
             }
         }
 
         if (dbEnum == null) 
             throw new KDbException(KDbException.FATAL,"Could not get DB enumeration.");
         return dbEnum;
     }//getDbEnumeration

     /**
      * Begin or reset a search of the db
      */ 
     public void resetSearch() throws KDbException {
        if (dbEnum == null)
            dbEnum = getDbEnumeration();
        else {
            dbEnum.rebuild();
            dbEnum.reset();
        }
     }
     /**
      *  Call this repeatedly to get all the records in a given store
      */
     public KDbRecord getNextRecord() throws KDbException {
        KDbRecord curRec = null;
        if (dbEnum == null)
            getDbEnumeration();
        
         if ((dbEnum != null) && (dbEnum.hasNextElement()) ){
             try {
                 byte[] curData = dbEnum.nextRecord();
                 if (curData != null) {
                     curRec = instantiateRecord(curData);
                 }
             }
             catch (InvalidRecordIDException invalidIDEx) {
                 if (bDebug) System.err.println("Unable to get next record in db: " + invalidIDEx);
             }
             catch (RecordStoreException storeEx) {
                 if (bDebug) System.err.println("Unable to get next record in db: "+ storeEx);
             }
           }
         
         return curRec;
     }//getNextRecordData
          
     
    /**
     * Remove a db entry
     **/
    public void removeEntryByID(int id) throws KDbException {
        try {
            openDB();
            dbStore.deleteRecord(id);
        }
        catch (Throwable t) { 
            if (bDebug)  {
            	System.err.println("removeEntryByID failed: " );
           	 	t.printStackTrace();
           	 }
            throw new KDbException(KDbException.RECOVERABLE, "Could not delete record: " + id);
        }
    }//removeEntryByID
    /**
     * Given an RMS record ID, return the entry 
     */
     public KDbRecord getEntryByID(int id) throws KDbException {
         KDbRecord curRec = null;
         
         try {
             openDB();//make sure db is open
            byte[] curData = dbStore.getRecord(id);
            if (curData != null) {
                curRec = instantiateRecord(curData);
            }
         }
         catch (java.lang.OutOfMemoryError oom)
         {
         	if (bDebug) System.err.println("Load record oom: "  + Runtime.getRuntime().freeMemory());  
  
         }
         catch (Throwable t) {
           if (bDebug) System.err.println("Loading record " + id + " failed with ex: " + t);  
         }
         
         if (curRec == null)
             throw new KDbException(KDbException.FATAL, "Could not load db record.");
         
         return curRec;
     }
    /** Creates a new instance of KPrefsAccessor if necessary */
    protected KDbAccessor() {
        dbStore = null;
        dbComparator = null;
        dbEnum = null;              
     }//KDbAccessor
    
    /**
     * Open the DB RecordStore-- create if necessary
     */
    protected boolean openDB() {
        if (dbStore == null) {
            //if (bDebug) System.out.println("Opening RecordStore: " + storeName);
            try {dbStore = RecordStore.openRecordStore(storeName,true);}
            catch (RecordStoreException storeEx) {
             if (bDebug) System.err.println("openDB storeEx: " + storeEx);   
            }
        }
 
        return (dbStore != null);
    }//openDB
    
    protected void closeDB() {
        if (dbStore != null) {
            try {
                dbStore.closeRecordStore();
            }
            catch (RecordStoreNotOpenException closedEx) {
                if (bDebug) System.err.println("closeDB not open: " +closedEx);
            }
            catch (RecordStoreException storeEx) {
                if (bDebug) System.err.println("closeDB storeEx: " +storeEx);
            }
            dbStore = null;
        }
    }//openDB
    

   /**
    *
    */
   protected KDbRecord instantiateRecord(byte[] srcData) {
        DataInputStream is = new DataInputStream(new ByteArrayInputStream(srcData));
        KDbRecord curRec = createBlankRecord();
        curRec.setRecID(KDbRecord.INVALID_REC_ID);
        try {curRec.readFromStream(is);}
        catch (IOException ioEx) {
           if (bDebug)  System.err.println("instRec ioEx: "+ ioEx);
        }
        catch (Exception ex) {
            if (bDebug) {
            System.err.println("instRec ex: "+ ex);           
            ex.printStackTrace();
            }
        }
        
        return curRec;
   }//instantiateRecord
   
   /**
    * Write this data to persistent object store.
    */ 
   public void makePersistent(KDbRecord rec) throws KDbException {
        boolean isNew = false;
        
        // //make sure DB is open...it might be already
        if (!openDB()) 
            throw new KDbException(KDbException.RECOVERABLE,"DB Could not be opened!");
        
        
        try {
            //we don't want any other thread writing to the store while we are 
            //writing to the store...sorry
            synchronized (dbStore) {  
                if (rec.getRecID() == KDbRecord.INVALID_REC_ID) {
                    isNew = true;
                    rec.setRecID(dbStore.getNextRecordID());
                }
                try {
                	
                	int totalSize = rec.guessRecordSize();
                	System.out.println("rec size: " + totalSize);
                    ByteArrayOutputStream baos = new ByteArrayOutputStream(totalSize);
        			DataOutputStream os = new DataOutputStream(baos);

	                rec.writeToStream(os);
               		os.close();


                	byte[] recData = baos.toByteArray();

	                if (isNew)  {
	                   // int recID = 
	                    	dbStore.addRecord(recData,0,recData.length);
	                    //if (bDebug) System.out.println("New record ID: " + recID+ "=" + rec.getRecID());
	                }
	                else { 
	                   // if (bDebug) System.out.println("Saving record ID: " + rec.getRecID());
	                    dbStore.setRecord(rec.getRecID(),recData,0,recData.length);
	                }
                }
                catch (java.lang.OutOfMemoryError oom)
                {
                	if (bDebug) System.err.println("--- OOM persist: "  + Runtime.getRuntime().freeMemory());
                }
                
            }//synchronized (dbStore)
        }
        catch (InvalidRecordIDException badIDEx) {
            if (bDebug) System.err.println("--- makePersist  badIDEx: " +badIDEx);
        }
        catch (Exception ex) {
            if (bDebug) System.err.println("--- makePersist failed: " +ex);
            throw new KDbException(KDbException.FATAL,"makePersistent ex: " + ex);
        }
   }//makePersistent
   
  
    
}//class KPrefsAccessor

