/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on November 1, 2002, 1:15 PM
 */

package org.kablog.kprefs;
import javax.microedition.rms.*;

import org.kablog.kdb.*;

import java.io.*;


/**
 *
 * @author  Todd C. Stellanova
 */
public abstract  class KPrefsDB {
    
    protected static KPrefsDB prefsCache;
    
    private static final boolean bDebug = false;
    
    public final static String PREFS_STORENAME = "prefs";
    protected RecordStore prefsStore;
     protected KDbRecord prefsData;
     public static final int INVALID_REC_ID = -666;
     protected int prefsRecID = INVALID_REC_ID;

    /**
     * Create a template blank prefs record.
     */
   protected abstract void loadBlankPrefs();
   
     /**
      * @return True if prefs exist, false otherwise.
      */
     public boolean prefsExist() {
        boolean bExists = false;
        
        KDbRecord curPrefs = getExistingPrefsRecord();
        if (curPrefs != null) 
            bExists = true;
        
        return bExists;
     }//prefsExist
     

     /**
      * Get the prefs record stored in RMS iff it exists.
      */
     public KDbRecord getExistingPrefsRecord() {
 
        openPrefs();
          
        if (prefsData == null) {
            
           try {
               int recCount = prefsStore.getNumRecords();
               if (recCount > 1) {
                //if (bDebug) System.out.println("num records: " +recCount);
                cleanupPrefsStore(recCount);
                recCount = prefsStore.getNumRecords();//recheck
                //if (bDebug) System.out.println("num records: " +recCount);
               }

               if (recCount > 0) {
                   RecordEnumeration recEnum = prefsStore.enumerateRecords(null,null,true);
                   if (recEnum.hasNextElement()) {
                       int recID = recEnum.nextRecordId();
                       //if (bDebug) System.out.println("Loading prefs record: " + recID);
                       try {
                            byte[] recData = prefsStore.getRecord(recID);
                           instantiatePrefsRecord(recData);
                           prefsRecID = recID;
                       }
                       catch (InvalidRecordIDException invalidIDEx) {
                        if (bDebug) System.err.println("getRecord badID: " +invalidIDEx);
                       }
                    }
               }
            }
            catch (Exception ex) {
                if (bDebug) System.err.println("Load prefs failed: " +ex);
            }
        }
        
        return prefsData;
     }//getExistingPrefsRecord
     
     
     /**
     * Get the prefs record stored in RMS, if it exists. If it
     * doesn't exist, create it.
     */ 
    public KDbRecord getPrefsRecordAlways() {
        
        KDbRecord prefs = getExistingPrefsRecord();
        if (prefs == null) {
          createDefaultPrefs();
          prefs = prefsData;
        }
        
        return prefs;
    }//getPrefsRecordAlways
    

    /**
     * Save changes to prefs, to the record store.
     */
    public void flushPrefs() {
        if (prefsData != null) {
            makePersistent(prefsData);
        }
    }//flushPrefs 
    
    /** Creates a new instance of KPrefsAccessor if necessary */
    protected KPrefsDB() {
        prefsStore = null;
        prefsData = null;
              
        boolean storeOpened = openPrefs();
        if (storeOpened) {
            
        }
     }//KPrefsDB
    
    
    protected boolean openPrefs() {
        if (prefsStore == null) {
            try {prefsStore = RecordStore.openRecordStore(PREFS_STORENAME,true);}
            catch (RecordStoreException storeEx) {
             if (bDebug) System.err.println("openPrefs storeEx: " + storeEx);   
            }
        }
        if (prefsStore != null) 
           return true;
        else 
           return false;
    }//openPrefs
    
    protected void closePrefs() {
        if (prefsStore != null) {
            try {
                prefsStore.closeRecordStore();
            }
            catch (RecordStoreNotOpenException closedEx) {
                if (bDebug) System.err.println("closePrefs closedEx: " +closedEx);
            }
            catch (RecordStoreException storeEx) {
                if (bDebug) System.err.println("closePrefs storeEx: " +storeEx);
            }
            prefsStore = null;
        }
    }//closePrefs
    
    /**
     * We only allow one prefs entry at a time-- blow away any others
     */
    protected void cleanupPrefsStore(int recCount) {
        if (recCount > 1) {
            //if (bDebug) System.err.println("cleanupPrefsStore: num records: " +recCount);
            try {
               closePrefs(); //close the prefs store before we modify it
               prefsStore.deleteRecordStore(PREFS_STORENAME);
               openPrefs();
               createDefaultPrefs();// create default prefs entry
            }
            catch (Exception ex) {
                if (bDebug) System.err.println("cleanupPrefs ex: " +ex);
            }
        }
  
    }//cleanupPrefsStore
    
 

   
   /**
     * Create a default prefs entry, in case there is no prefs entry yet.
     * Insert this entry into the database.
     */
    protected void createDefaultPrefs() {
        loadBlankPrefs();
        makePersistent(prefsData);       
    }//createDefaultPrefs
   
   protected void instantiatePrefsRecord(byte[] srcData) {
        DataInputStream is = new DataInputStream(new ByteArrayInputStream(srcData));
        if (prefsData == null) 
            loadBlankPrefs();
        try {prefsData.readFromStream(is);}
        catch (IOException ioEx) { 
            if (bDebug) System.err.println("instPrefs ioEx: " +ioEx); 
        }
   }//instantiatePrefsRecord
   
   /**
    * Write this data to persistent object store.
    */ 
   protected void makePersistent(KDbRecord rec) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream os = new DataOutputStream(baos);
        
        try {
            prefsData.writeToStream(os);
            os.close();

            byte[] recData = baos.toByteArray();
         
            if (prefsRecID ==INVALID_REC_ID)  {
                prefsRecID = prefsStore.addRecord(recData,0,recData.length);
                //if (bDebug) System.out.println("New record ID: " + prefsRecID);
            }
            else { 
                //if (bDebug) System.out.println("Saving record ID: " + prefsRecID);
                prefsStore.setRecord(prefsRecID,recData,0,recData.length);
            }
        }
        catch (InvalidRecordIDException badIDEx) {
            if (bDebug) System.err.println("makePersist badIDEx: " +badIDEx);
        }
        catch (Exception ex) {
            if (bDebug) System.err.println("makePersist ex: " +ex);
        }
   }//makePersistent
   
  
    
}//class KPrefsAccessor

