/* Copyright (c) 2001-2005 Todd C. Stellanova, rawthought
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The  above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE. 
 * 
 *
 * Created on October 28, 2002, 2:04 PM
 */
package org.kablog.midlet2;


import java.util.*;
import java.io.*;

import org.kablog.kdb.*;
import org.kablog.kgui.*;
import org.kablog.xmlrpc.*;




/**
 * A class for maintaining state of a connection with the remote XML-RPC blog server.
 * This class calls synchronously into the XMLRPC client:
 * callers should deal with showing UI status and so forth in a separate thread.
 * 
 * @author  Todd C. Stellanova
 */
public class KablogEngine implements KNetworkClient {
         protected final static boolean bDebug = false;
       
         //This is the Blogger app key provided for KABLOG in general: right now
         //both the Series 60 version and the J2ME version share the same appkey.
         //If this becomes a problem in the future, we may need separate keys.
       	private static final String sAppKey = "D356E2500FAD10C5664D07CC8790E7A807776BE3";
        private Vector callParms;
        protected KablogPrefsDB prefsDB;
        
        protected KXmlRpcClient2 netClient;
        protected EntriesDB entriesDB;
         
        //there are three general levels of support:
        //best: Full MovableType 2.6 or above support
        //good: MetaWeblog support
        //okay: blogger API support only
        
        //asynchronous operations 
        protected int   topLevelProgress;
        protected Object    asyncResult;
        protected Object    asyncParms;
        protected int       asyncOpType;
        
        //published async method codes
        public static final int POST_SIMPLE = 2; //post a simple body-only entry
        public static final int POST_ENTRY = 4; //post one EntryRecord
        public static final int POST_MULT = 8; //post multiple entries
        public static final int GET_BLOGSLIST = 16; //get the list of blogs the user can access
        public static final int GET_SUPP_METHODS = 32; //get a list of supported xmlrpc methods
        
        public KablogEngine() {
            prefsDB = KablogPrefsDB.getDB();
         }
        
       
        /**
         * Set up the entries db, if we already have one set up.
         */ 
        public void setEntriesDB(EntriesDB db) {
         entriesDB = db;   
        }//setEntriesDB
        

        /**
         * Get access to the entries DB
         */
        protected void initEntriesDB() {
            if (this.entriesDB == null) {
                entriesDB = EntriesDB.getDB();
            }
        }//initEntriesDB
        
        /**
         * Set up the KXmlRpcClient 
         * @param the URL to use to access the XMLRPC server
         */
        protected void initNetClient(String targetURL) {
            if (netClient == null) {
                //create new KXmlRpcClient
                netClient = new KXmlRpcClient2(targetURL);
            }
            else {
                //reuse existing KXmlRpcClient
                netClient.setTargetURL(targetURL);
            }
        }//initNetClient
        
        
        /**
         * Post multiple entries, and optionally remove them from the db when
         * posted successfully.
         * @param 
         * @param Delete entries from local cache DB when finished
         */
        public void postMultipleEntries(Vector entryIdList, boolean deleteAfterPost) {
            initEntriesDB();
             
            for (int i = 0; i < entryIdList.size(); i++) {
                try {
                    Integer recID = (Integer)entryIdList.elementAt(i);
                    if (bDebug) System.out.println("loading entry ID: " + recID);
                    EntryRecord curEntry = (EntryRecord)entriesDB.getEntryByID(recID.intValue());
                    if (bDebug) System.out.println("...loading done.");
                    postEntry(curEntry);
                    entriesDB.removeEntryByID(curEntry.getRecID());
                }
                catch (KDbException dbEx) {
                     if (bDebug) System.err.println("postMult dbEx: " + dbEx);
                    break; //quit trying to post entries at this point!
                }
                catch (IOException ioEx) {
                     if (bDebug) System.err.println("postMult ioEx: " + ioEx);
                    asyncResult = ioEx;
                    break; //quit trying to post entries at this point!
                }
                catch (KXmlRpcFault fault) {
                     if (bDebug) System.err.println("postMult fault: " + fault);
                    asyncResult = fault;
                    break; //quit trying to post entries at this point!
                }
            }
        }//postMultipleEntries
        
        /**
         * Post an entry including a title and creation date
         * @param The entry to be posted to the blog server
         */
         public void postEntry(EntryRecord entry) throws IOException, KXmlRpcFault {
        	 
            Date creationDate = KDateConverter.stringToDate(entry.sCreationDate,KDateConverter.DATE_TIME);
             KablogPrefs prefs = prefsDB.getKablogPrefsRecord();
             
            if (bDebug)
            	System.out.println("postEntry: " +entry.sTitle+ ":\n"+ entry.sBody);
            
            String entryBody = null;
           
            if ((prefs.nBlogServerType == prefs.SERVTYPE_MOV_TYP) ||
                    (prefs.nBlogServerType == prefs.SERVTYPE_METAWEB)) 
            {
            
                if (entry.mediaData != null) {
                    if ((entry.mediaURL == null) ||  (entry.mediaURL.length() < 3)  ) {
                        //need to post the media to the server
                       if (bDebug) 
                    	   System.out.println("mediaName: " +entry.mediaName);
                       
                       // By default we always post media to the "media/" directory:
                       // this centralizes the location of all blog media files.  Otherwise,
                       // Movable Type &co. will put the media files in some arbitrary place.
                       Object mediaResult = this.postMediaObject(entry.mediaData,entry.mediaType,"media/" + entry.mediaName);
 
                       if (mediaResult instanceof String) {
                           entry.mediaURL = (String)mediaResult;
                       }
                       else {
                           //TODO: FAILED
                            if (bDebug) 
                            	System.err.println("Failed to get media URL!");
                           throw new KXmlRpcFault(netClient.getFailureCode(), "Couldn't upload media.");                         
                           //proceed no further
                       }
                    }
                    
                    if (entry.mediaURL != null) {
                    
                    	String phoneModel = System.getProperty("microedition.platform");
                    	if (null == phoneModel) phoneModel = "Generic_Phone";
                    	
                    	int firstSlashPos = phoneModel.indexOf("/");
                    	if (-1 != firstSlashPos)
                    	{
                    		phoneModel = phoneModel.substring(0,firstSlashPos);
                    	}
                    	
                        entryBody = "<img src=\"" + entry.mediaURL 
                           + "\">\n" 
                           + entry.sBody;
                    }
                }
                else {
                    initNetClient(prefs.sURL);
                    entryBody = entry.sBody;
                }
             
                if (bDebug) System.out.println(  "\n entryBody: " + entryBody);
				 
                         
 
                
                  
                   //build the content struct
                Hashtable content = new Hashtable(5);
                content.put("title", entry.sTitle);
                content.put("description", entryBody);
                content.put("dateCreated",creationDate);
                if (prefs.nBlogServerType == prefs.SERVTYPE_MOV_TYP) {
                        content.put("mt_allow_comments", 
                                new Boolean(entry.bAllowComments));// settable by user
                        content.put("mt_convert_breaks", 
                                new Boolean(entry.bConvertBreaks));// settable by user
                }


                callParms  = new Vector(5);
               callParms.addElement(prefs.sCurBlogID);
                callParms.addElement(prefs.sUsername);
                callParms.addElement(prefs.sPassword);
                callParms.addElement(content);
                callParms.addElement(new Boolean(entry.bPublish));  //settable by user
		//System.out.println("bPublish parm: " + ((Boolean)callParms.elementAt(4)) );

                //if (bDebug) System.out.println("Calling KXmlRpcClient.remoteCall... " + prefs.sURL);
                Object retVal = netClient.remoteCall(netClient.RPCNAME_POST_FULL_ENTRY,callParms);
                if (bDebug) System.out.println("remoteCall result: " + retVal);

                callParms = null;
            }
            else {
                initNetClient(prefs.sURL);
                //System.out.println("nBlogServerType: " + prefs.nBlogServerType);
                postNewSimpleEntry(entry,prefs);
            }
        }//postEntry
 
        
        /**
         * Post a new simple blog entry (body only, no title and so forth)
         * using the connection info from prefs.
         */
        public void postNewSimpleEntry(EntryRecord entry, KablogPrefs prefs) throws IOException, KXmlRpcFault {
            // if (bDebug) System.out.println("postNewSimpleEntry: " + entry.sBody);
            callParms  = new Vector(6);
             
            callParms.addElement(sAppKey);
            callParms.addElement(prefs.sCurBlogID);
            callParms.addElement(prefs.sUsername);
            callParms.addElement(prefs.sPassword);
            callParms.addElement(entry.sTitle + "\n\n" + entry.sBody);
            callParms.addElement(new Boolean(entry.bPublish));  
            //System.out.println("bPublish parm: " + ((Boolean)callParms.elementAt(5)) );
            
           // if (bDebug) System.out.println("remoteCall... " + prefs.sURL);
           // Object retVal = 
            	netClient.remoteCall(netClient.RPCNAME_POST_SIMPLE_ENTRY,callParms);
            //if (bDebug) System.out.println("Result: " + retVal);
          
            callParms = null;
            
        }//postNewSimpleEntry
        
        
/**
metaWeblog.newMediaObject
Description: Uploads a file to your webserver. 
Parameters: String blogid, String username, String password, struct file
Return value: URL to the uploaded file.
Notes: the struct file should contain two keys: base64 bits (the base64-encoded contents of the file) and String name (the name of the file). The type key (media type of the file) is currently ignored.
*/
         public Object postMediaObject(byte[] rawData, String mimeDataType, String objectName) 
            throws IOException, KXmlRpcFault 
         {
             Object retVal = null;
             KablogPrefs prefs = prefsDB.getKablogPrefsRecord();
            if (bDebug) 
            	System.out.println("== postMediaObject: " + objectName + " (" + mimeDataType + "): " + rawData.length);
            initNetClient(prefs.sURL);
            
            //we only allow posting of media objects if the server supports it!
            if ((prefs.nBlogServerType == prefs.SERVTYPE_MOV_TYP) ||
                    (prefs.nBlogServerType == prefs.SERVTYPE_METAWEB)) {
                     
                //build the content struct
                //<bits>, <name>, <type>
            	Hashtable content = new Hashtable(3);
                content.put("bits",rawData);
                content.put("name",objectName);
                content.put("type",mimeDataType);
                
                callParms  = new Vector(4);
                callParms.addElement(prefs.sCurBlogID);
                callParms.addElement(prefs.sUsername);
                callParms.addElement(prefs.sPassword);
                callParms.addElement(content);
                
                if (bDebug) 
                	System.out.println("=== calling remoteCall...");
                retVal = netClient.remoteCall(netClient.RPCNAME_POST_MEDIA_OBJ,callParms);

                if (retVal instanceof String) {
                   if (bDebug) 
                	   System.out.println("=== newMediaObject result: " + (String)retVal);
                }
                else if (retVal instanceof Hashtable) {
                    //struct returned
                    String newURL = (String)((Hashtable)retVal).get("url");
                    if (bDebug) 
                    	System.out.println("=== newMediaObject url: " + newURL);
                    retVal = newURL;
                }
                else {
                    int rc = 0;
                    if (retVal == null) {
                        rc = netClient.getFailureCode();
                         if (bDebug) 
                        	 System.err.println("=== newMediaObject rc: " + rc );
                    }
                }

                callParms = null;
            }
            
            return retVal;

        }//postMediaObject

        /**
         *
         */
        public void getUsersBlogsListAsync() {
            
            
            try {
                asyncResult = getUsersBlogsList();
            }
            catch (KXmlRpcFault fault) {
                asyncResult = fault;
                 if (bDebug) System.err.println("getBlogs fault: " + asyncResult);
            }
            catch (IOException ioEx) {
                asyncResult = ioEx;
                 if (bDebug) System.err.println("getBlogs ioEx: " + asyncResult);
            }
            
        }//getUsersBlogsListAsync
        
        /**
         * Get the list of blogs for which this user has write privileges.
         * @param sUsername  The user's username. (e.g. "tstellanova" in mt)
         * @param sPassword  The user's password (in cleartext).
         */
        protected Vector getUsersBlogsList() throws IOException, KXmlRpcFault {
            KablogPrefs prefs = prefsDB.getKablogPrefsRecord();
            //if (bDebug) System.out.println("getBlogsList prefs: " + prefs);
            callParms  = new Vector(3);
            initNetClient(prefs.sURL);
            
            callParms.addElement(sAppKey);
            callParms.addElement(prefs.sUsername);
            callParms.addElement(prefs.sPassword);

            //if (bDebug) System.out.println("remoteCall... " + prefs.sURL);
            Vector blogList = (Vector)netClient.remoteCall(netClient.RPCNAME_GET_USERS_BLOGS,callParms);
             /*
            if ((bDebug) && (blogList != null))  {
                for (int i = 0; i < blogList.size(); i++) {
                    Hashtable blog =  (Hashtable)blogList.elementAt(i);
                    String blogName = (String)blog.get("blogName");
                    String blogURL = (String)blog.get("url");
                    String blogID = (String)blog.get("blogid");

                   // System.out.println("### Blog '" +blogName+ "' id: " +blogID+ " url: "+blogURL);
                }
            }
              */
            
            callParms = null;
            
            return blogList;
        }//getUsersBlogsList

         /**
         * Get the list of XMLRPC methods the blog server supports
         */
        /*
        public Vector getSupportedMethods() throws IOException, KXmlRpcFault {
            KablogPrefs prefs = prefsDB.getKablogPrefsRecord();
            //if (bDebug) System.out.println("getBlogsList prefs: " + prefs);
            callParms  = null;
            initNetClient(prefs.sURL);
            String methDump = "Methods: ";

            //if (bDebug) System.out.println("remoteCall... " + prefs.sURL);
            Vector methsList = (Vector)netClient.remoteCall("mt.supportedMethods",callParms);
             
            
            for (int i = 0; i < methsList.size(); i++) {
                String meth = (String)methsList.elementAt(i);
                if (meth != null) {
                    System.out.println("Method: " + meth);
                    methDump += meth + "\n";
                }
            }
             
            callParms = null;
            return methsList;


        }//getSupportedMethods
        */
        
        
     //Methods for supporting KNetworkClient interface
            
     /**
     * @return The progress of this item, from 0 to 100 
     */
    public int getProgress() {   
     if (netClient != null) {
         if (topLevelProgress > 0) return topLevelProgress;
         else return netClient.getProgress();
     }
     else 
         return 0;//TODO update with valid progress value?
    }//getProgress
    
    
    /**
     * @ return Zero if no failure, or the nonzero error code;
     */
    public int getFailureCode() {
        int fcode = 0;
        
        if (this.asyncResult != null) {
            if (asyncResult instanceof KXmlRpcFault) {
                fcode  = ((KXmlRpcFault)asyncResult).code;
                if (fcode == KXmlRpcFault.UNAUTHORIZED)
                    fcode = KProgressStatus.ERR_BAD_USER;
                
 
            }
            else if (asyncResult instanceof Exception) {
                fcode = KProgressStatus.ERR_IO;
            }           
        }
        else if (netClient != null) {
            fcode = netClient.getFailureCode();
        }
 
        return fcode;
    }//getFailureCode
    
    
    /**
     * @return True if operation can be cancelled, false if not
     */
    public boolean isCancellable() {
     return false; //none of the operations can be cancelled currently 
    }//isCancellable
        
    
     /**
     * Set up an asynchronous operation to execute as soon as
     * this client's run method is called.
     * @param The type of operation to execute. This will be implementation-specific.
     * @param Parameters to help execute the asynchronous operation
     */
    public void setupAsyncOp(int type, Object parms) {
        //System.out.println("setupAsyncOp: " + type);
        asyncOpType = type;
        asyncParms = parms;
        asyncResult = null;
        topLevelProgress = 0;
    }
    
    /**
     * Run the current network-bound operation (in a separate thread).
     */ 
    public void run() {
        //System.out.println("KablogEngine run...");
        
        switch (asyncOpType) {
           case (POST_MULT):{
               //System.out.println("KablogEngine posting...");
               Vector idList = (Vector) (((Vector)asyncParms).elementAt(0));
               boolean doDeleteAfterPost = ((Boolean)((Vector)asyncParms).elementAt(1)).booleanValue(); 
               postMultipleEntries(  idList,doDeleteAfterPost);
               topLevelProgress = 100;
            }//POST_MULT
            break;
               
            case (GET_BLOGSLIST):
               //System.out.println("### getting blogs list...");
                getUsersBlogsListAsync();
                //System.out.println("### getUsersBlogsListAsync returned: " + asyncResult);
                topLevelProgress = 100;
                break;
        }
    }//run
     
    public void reset() {
       // System.out.println("### engine.reset!");
        netClient.reset();
        asyncOpType = 0;
        asyncParms = null;
        asyncResult = null;
    }
    
    public void cancel() { //XXX No op for now?
       if ((netClient != null) && netClient.isCancellable())
           netClient.cancel();
    }//cancel
        
    
    /**
     *
     */
    public Object getAsyncResult() {
        //System.out.println("### getAsyncResult: " + asyncResult);
        return asyncResult; 
    }    

 
}// class KablogEngine
